
#include "general.h"
#include "P_Motion.h"
#include "function.h"
#include "function.icc"

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
//
// Initialization and destructors of PolarMotion class.
//
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

//----------------------------------------------------------------------
// 
//----------------------------------------------------------------------

PolarMotion::PolarMotion( float dist , float ang , float rot ) :
  _distance(dist),
  _angle(ang),
  _rotation(rot)
{

}

//----------------------------------------------------------------------
// Computes the motion between the two points. 
//----------------------------------------------------------------------
PolarMotion::PolarMotion( State start, State end)
{
  if ( start == end) {
    _distance = 0.0;
    _angle = 0.0;
    _rotation = 0.0;
  }
  else {
    State diff = end - start;
    
    _distance = diff.length();
    
    _angle = (float) atan2( diff[Y_DIM], diff[X_DIM]) - start[ROT_DIM];

    _rotation = diff[ROT_DIM];
  }
}

//----------------------------------------------------------------------
// Sets the parameters.
//----------------------------------------------------------------------

void
PolarMotion::set( float dist, float ang, float rot)
{
  _distance = dist;
  _angle    = ang;
  _rotation = rot;
}

//----------------------------------------------------------------------
// Some access functions.
//----------------------------------------------------------------------

float
PolarMotion::distance() const
{
  return _distance;
}


float
PolarMotion::rotation() const
{
  return _rotation;
}


float
PolarMotion::angle() const
{
  return _angle;
}

//----------------------------------------------------------------------
// Returns the point reached after the motion starting at start.
//----------------------------------------------------------------------
 State
PolarMotion::endPoint( const State& start) const
{
  State end(start);

  apply(end);

  return end;
}

//----------------------------------------------------------------------
// Sets the point reached after the motion starting at start.
//----------------------------------------------------------------------
 void
PolarMotion::apply( State& s) const
{
  float absoluteAngle = s.value(ROT_DIM) + _angle;

  s[X_DIM]   += cos(absoluteAngle) * _distance;
  s[Y_DIM]   += sin(absoluteAngle) * _distance;
  s[ROT_DIM]  = normalizedAngle( s[ROT_DIM] + _rotation);
}


//----------------------------------------------------------------------
// Sets the point reached after the motion starting at start.
//----------------------------------------------------------------------
 void
PolarMotion::apply( State& s, float fraction) const
{
    float absoluteAngle = s.value(ROT_DIM) + _angle * fraction;
    float shortDist = _distance * fraction;

    s[X_DIM]   += cos(absoluteAngle) * shortDist;
    s[Y_DIM]   += sin(absoluteAngle) * shortDist;
    s[ROT_DIM] += _rotation * fraction;
    s.normalizeOrientation();
}




