#ifndef STATE_LOADED
#define STATE_LOADED

#include "function.h"

#define NUMBER_OF_DIMENSIONS 3
const int X_DIM    = 0;
const int Y_DIM    = 1;
const int ROT_DIM  = 2;


/////////////////////////////////////////////////////////////////
// This class represents the state to be estimated.
// Everything is public since the objects are capsulated by higher
// level objects.
/////////////////////////////////////////////////////////////////
class State {
public:
  State();
  State( const State& s);                   // Copy other state
  State( float x, float y, float orientation);  

  virtual ~State();

  // Some operators.
   State &  operator  = (const State & s);
   State    operator  - (const State & s) const;
   float &  operator [] (int dim);
   bool     operator != (const State& s) const;
   bool     operator == (const State& s) const;
//friend ostream& operator << (ostream& os, const State& state);
   State    operator + (const State& s) const;

  // Special handling of angles.
   void     normalizeOrientation();
   void     setOrientation( float angle);

  // Access functions.
   const float&  value( int dimension) const;
   int dimension() const;
  
  // Treat the state as if it represented the edges of a hyper cube.
   float    volume() const;
   float    length() const;

   void     setRandom( const State& min, const State& max);
  
   void     addNoise( const State& stdDev); // Add wite noise to the state.

  // Range checking for states.
   bool     inBoundingBox( const State& minCorner, const State& maxCorner) const;
   bool     largerOrEqual( const State& s)  const;
   bool     smallerOrEqual( const State& s) const;

  friend class SampleSet;
  
protected:
  float _value[NUMBER_OF_DIMENSIONS];
};


#endif

















