
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "allocate.h"
#include "general.h"
#include "State.h"


template<class T>
void
free2D( T **field, int dim)
{
  if ( ! field)
    return;
  
  for ( int i = 0; i < dim; i++)
	free(field[i]);
  free(field);
}

template<class T>
void
allocate1D( T** dest, int dim)
{
  T* pt;
  if ( (pt = (T*) malloc( dim * sizeof(T))) == NULL) {
    fprintf( stderr,
	     "Error! Not enough memory for allocating in <allocate1D>.\n");
    fprintf( stderr, "Desired dimension: %d\n", dim);
    exit(0);
  }
  *dest = pt;
}


template<class T>
void
allocate2D( T*** dest, int dim1, int dim2)
{
  T** pt;
  
  if ( (pt = (T**) malloc( dim1 * sizeof( T*))) == NULL) {
    fprintf( stderr,
	     "Error! Not enough memory for allocating in <allocate2D>.\n");
    fprintf( stderr, "Desired dimensions: %d %d.\n", dim1, dim2);
    exit(0);
  }
  for ( int x = 0; x < dim1; x++)
    if ( (pt[x] = (T*) malloc( dim2 * sizeof(T))) == NULL) {
      fprintf( stderr,
	       "Error! Not enough memory for allocating in <allocate2D>.\n");
      fprintf( stderr, "Desired dimensions: %d %d.\n", dim1, dim2);
      exit(0);
    }
  *dest = pt;
}


template<class T>
void
allocate3D( T**** dest, int dim1, int dim2, int dim3)
{
  T*** pt;

//    fprintf(stderr, "# Allocate 3D memory: %d %d %d\n",
//  	  dim1, dim2, dim3);
  
  if ( (pt = (T***) malloc( dim1 * sizeof( T**))) == NULL) {
    fprintf( stderr,
	     "Error! Not enough memory for allocating in <allocate3D>.\n");
    fprintf( stderr, "Desired dimensions: %d %d %d.\n", dim1, dim2, dim3);
    exit(0);
  }
  for ( int x = 0; x < dim1; x++) {
    if ( (pt[x] = (T**) malloc( dim2 * sizeof(T*))) == NULL) {
      fprintf( stderr,
	       "Error! Not enough memory for allocating in second loop of <allocate3D>.\n");
      fprintf( stderr, "Desired dimensions: %d %d %d.\n", dim1, dim2, dim3);
      exit(0);
    }
    for ( int y = 0; y < dim2; y++) {
      if ( (pt[x][y] = (T*) malloc( dim3 * sizeof(T))) == NULL) {
	fprintf( stderr,
		 "Error! Not enough memory for allocating in in third loop of <allocate3D>.\n");
	fprintf( stderr, "Desired dimensions: %d %d %d.\n", dim1, dim2, dim3);
	exit(0);
      }
    }
  }
  *dest = pt;
}


template<class T>
void
free3D( T ***field, int dim1, int dim2)
{
  if ( ! field)
    return;
  
  for ( int i = 0; i < dim1; i++) {
    for ( int j = 0; j < dim2; j++)
      free(field[i][j]);
    free(field[i]);
  }  
  free(field);
}


template <class T>
void
copyAllocate( T** src, T** dest, int dim1, int dim2)
{
  // First allocate.
  allocate2D( & dest, dim1, dim2);

  // Now copy.
  for ( int x = 0; x < dim1; x++)
    for ( int y = 0; y < dim2; y++)
      dest[x][y] = src[x][y];
}


//----------------------------------------------------------------------
// Instantiations.
//----------------------------------------------------------------------

template void copyAllocate( float** src, float** dest, int dim1, int dim2);

template void allocate1D( float** dest, int dim);
template void allocate1D( int** dest, int dim);

template void allocate2D( float*** dest, int dim1, int dim2);
template void allocate2D( int*** dest, int dim1, int dim2);
template void allocate2D( bool*** dest, int dim1, int dim2);

template void free2D( float** field, int dim);
template void free2D( int** field, int dim);

template void allocate3D( unsigned char**** dest, int dim1, int dim2, int dim3);
template void allocate3D( float**** dest, int dim1, int dim2, int dim3);
template void free3D( unsigned char***field, int dim1, int dim2);
template void free3D( float***field, int dim1, int dim2);


