
/***************************************************************************
 ***************************************************************************
 ***************************************************************************
 *****
 *****
 ***** File:                        main.cc
 *****
 ***** Part of:                     RHINO SOFTWARE
 *****
 ***** Creator:                     Dieter Fox, Carnegie Mellon University
 *****
 ***** Date of creation:            July 1996
 *****
 *****
 *****
 *****
 ***** $Source: /home/rhino/rhino/CVS/bee/src/sampling/function.cc,v $
 *****
 ***** $Revision: 1.3 $
 *****
 ***** $Date: 2000/10/30 17:46:39 $
 *****
 ***** $Author: fox $
 *****
 *****
 *****
 ***** Contact dfox@cs.cmu.edu
 *****
 ***************************************************************************
 ***************************************************************************
 ***************************************************************************
 ***** 
 ***** THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY
 ***** APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE
 ***** COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS"
 ***** WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,
 ***** INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 ***** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE
 ***** RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
 ***** SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
 ***** NECESSARY SERVICING, REPAIR OR CORRECTION.
 ***** 
 ***** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
 ***** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY
 ***** MODIFY AND/OR REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE
 ***** LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL,
 ***** INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR
 ***** INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
 ***** DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU
 ***** OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY
 ***** OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN
 ***** ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *****
 ***************************************************************************
 ***************************************************************************
 ***************************************************************************
 *
 *
 *                  ----- REVISION HISTORY -----
 *
 * $Log: function.cc,v $
 * Revision 1.3  2000/10/30 17:46:39  fox
 * Multimapping!
 *
 * Revision 1.2  1999/09/09 02:48:40  fox
 * Final version before germany.
 *
 * Revision 1.1.1.1  1999/05/12 21:51:28  fox
 * Reorganization of the software in order to distinguish between general
 * sampling and localization.
 * ----------------------------------------------------------------------
 * automatically CVS: CVS:
 * ----------------------------------------------------------------------
 *
 * Revision 1.4  1999/04/19 22:40:42  fox
 * Minor changes.
 *
 * Revision 1.3  1999/03/16 07:54:25  fox
 * Added samples as observation. The samples are automatically integrated
 * into the sample set of each PositionEstimator. The different estimators
 * share the same display when being invoked by MultiLocalize. First attempt
 * to implement resampling (still need to check the different flags).
 *
 * Revision 1.2  1999/03/04 16:52:12  fox
 * Added first graphics stuff.
 *
 * Revision 1.1.1.1  1999/03/03 21:58:00  fox
 * New version of sampling based state estimation.
 *
 *
 *
 *
 ***************************************************************************
 ***************************************************************************
 ***************************************************************************/


//extern "C" {
#include <math.h>
//}

#include "general.h"
#include "function.h"


string
numberToString( int digits, int number)
{
  char* num = new char(digits + 1);
  for ( int i = 0; i < digits; i++)
    num[i] = '0';

  int sizeOfNumber = 1;
  int comparison = 10;
  while ( comparison <= number) {
    comparison *= 10;
    sizeOfNumber++;
  }
  char* startOfNumber = &(num[digits-sizeOfNumber]);
  sprintf( startOfNumber, "%d", number);

  string str( num);
  free(num);
  
  return str;
}


float
randomGauss()
{
  static int iset = 0;
  static float gset;
  float fac, rsq, v1, v2;
  if(iset == 0) {
    do {
      v1 = 2.0 * rand()/(RAND_MAX+1.0) - 1.0;
      v2 = 2.0 * rand()/(RAND_MAX+1.0) - 1.0;         
      rsq = v1*v1 + v2*v2;
    } while(rsq >= 1.0 || rsq == 0.0);
    fac = sqrt(-2.0*log(rsq)/rsq);
    gset = v1*fac;
    iset = 1;
    return v2*fac;
  }
  else {
    iset = 0;
    return gset;
  }
}


float
gauss( float x, float sigma, float mean)
{
  float tmp;
    
  if (sigma != 0.0) {
    tmp = exp( - 0.5 * sqr((mean-x) / sigma));
    return( tmp / sqrt( 2.0 * M_PI) / sigma);
  }
  else {
    if (x == mean)
      return 1.0;
    else
      return 0.0;
  }
    
}

/*****************************************************************************
 * BinarySearch for smallest k for which c(k)>=r
  *****************************************************************************/
int
binarySearch( float* values, float searchVal, int numberOfValues)
{
  int left = 0, right = numberOfValues - 1;
  int middle = numberOfValues / 2;

  if (values[left] > searchVal) return left;

  while(true) {
    if( values[middle] < searchVal) {
      if((right - middle) == 1) {
	// fprintf( stderr, "RETURN RIGHT %d %d %d\n", left, middle, right);
	return right;
      }
      left = middle;
      middle = middle + (right - middle) / 2;
      // fprintf( stderr, "LEFT %d %d %d\n", left, middle, right);
    }
    else {
      if((middle - left) == 1) {
	// fprintf( stderr, "RETURN LEFT %d %f %f\n", values[left] < searchVal ? middle : left,
	// values[left], searchVal);
	return values[left] < searchVal ? middle : left;
      }
      right = middle;
      middle = middle - (middle - left) / 2;
      // fprintf( stderr, "RIGHT %d %d %d\n", left, middle, right);
    }
  }
}
